package com.hzya.frame.webapp.entrance.service;

import com.hzya.frame.sys.entity.EsbReturnEntity;
import com.hzya.frame.sys.file.upload.entity.FileResultEntity;
import com.hzya.frame.sys.file.upload.entity.FileUploadDto;
import com.hzya.frame.web.entity.JsonResultEntity;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import java.lang.reflect.InvocationTargetException;

/**
 * @Content
 * @Author 👻👻👻👻👻👻👻👻👻👻 gjh
 * @Date 2022-12-07 8:47
 * @Version 1.0
 */
public interface IEntranceService {

    JsonResultEntity doBusiness(ServletRequest servletRequest, ServletResponse servletResponse) throws Exception;

    EsbReturnEntity doTstsBusiness(ServletRequest servletRequest, ServletResponse servletResponse);

    EsbReturnEntity doESbBusiness(ServletRequest servletRequest, ServletResponse servletResponse) throws Exception ;
    /***
     * 系统框架附件上传接口
     * @content:
     * @author 👻👻👻👻👻👻👻👻 gjh
     * @date  2023-10-08 15:36
     * @param
     * @return com.hzya.frame.web.entity.JsonResultEntity
     **/
    JsonResultEntity fileUpload(MultipartFile file, FileUploadDto entity, ServletRequest servletRequest, ServletResponse servletResponse) throws Exception;

    JsonResultEntity pluginfileUpload(MultipartFile file, FileUploadDto entity,String pluginPackageName, ServletRequest servletRequest, ServletResponse servletResponse) throws Exception;
    JsonResultEntity pluginZipUpate(FileUploadDto fileUploadDto) throws Exception;
    /***
     * 系统框架附件上传接口
     * @content:
     * @author 👻👻👻👻👻👻👻👻 gjh
     * @date  2023-10-08 15:36
     * @param
     * @return com.hzya.frame.web.entity.JsonResultEntity
     **/
    JsonResultEntity fileUpload(MultipartFile file, FileResultEntity entity, ServletRequest servletRequest, ServletResponse servletResponse) throws Exception;
    //平台接口
    Object platformInterface(ServletRequest servletRequest, ServletResponse servletResponse) throws InvocationTargetException, IllegalAccessException;
}
